

#include <stdint.h>
#include "interrupt.H"
#include "Timer.H"
#include "AlarmControl.H"


using namespace std;
using namespace avelino;


class MyListener : public TimerListener, public AlarmControlIO {
	protected:
		AlarmControl alarmControl;
    public:
        MyListener();
        virtual void timerExpired();
		virtual bool getAlarm();
		virtual void startTrigSound();
		virtual void stopTrigSound();
		virtual void startTrigSilence();
		virtual void stopTrigSilence();
};


#define  RCC_APB2PCENR  *((uint32_t *) 0x40021018)
#define  GPIOD_CFGLR    *((uint32_t *) 0x40011400)
#define  GPIOD_OUTDR    *((uint32_t *) 0x4001140C)
#define  GPIOA_CFGLR    *((uint32_t *) 0x40010800)
#define  GPIOA_OUTDR    *((uint32_t *) 0x4001080C)
#define  GPIOC_CFGLR    *((uint32_t *) 0x40011000)
#define  GPIOC_INDR     *((uint32_t *) 0x40011008)


MyListener::MyListener() : alarmControl(*this) {
	// enable clock on GPIOD
	RCC_APB2PCENR |= (((uint32_t) 1) << 5);
	// led on PD6
	GPIOD_CFGLR = (GPIOD_CFGLR & 0xF0FFFFFF) | 0x02000000;   // PD6 = output push-pull
	GPIOD_OUTDR &= ~(((uint32_t) 1) << 6);                   // PD6 <-- 0
	// enable clock on GPIOA
	RCC_APB2PCENR |= (((uint32_t) 1) << 2);
	GPIOA_CFGLR = (GPIOA_CFGLR & 0xFFFFF00F) | 0x00000660;          // PA1 and PA2, open drain output (for trig sound and trig silence)
	GPIOA_OUTDR |= (((uint32_t) 1) << 1) | (((uint32_t) 1) << 2);   // reset to PA1 = PA2 = 1 = high-z (because open drain output)
	// enable clock on GPIOC
	RCC_APB2PCENR |= (((uint32_t) 1) << 4);
	GPIOC_CFGLR = (GPIOC_CFGLR & 0xFFFFFF0F) | 0x00000040;          // PC1, floating input (for alarm signal read). Must be PC1 because it is 5 volt tolerant (or any other 5 volt tolerant pin)
}


void MyListener::timerExpired() {
	// GPIOD_OUTDR ^= (((uint32_t) 1) << 6);     debug blinker (comment for production use)
	this->alarmControl.run();
}


bool MyListener::getAlarm() {
	return (GPIOC_INDR & (((uint32_t) 1) << 1)) ? true : false;     // read PC1
}


void MyListener::startTrigSound() {
	GPIOA_OUTDR &= ~(((uint32_t) 1) << 1);      // PA1 = 0
}


void MyListener::stopTrigSound() {
	GPIOA_OUTDR |= (((uint32_t) 1) << 1);      // PA1 = high z
}


void MyListener::startTrigSilence() {
	GPIOA_OUTDR &= ~(((uint32_t) 1) << 2);      // PA2 = 0
}


void MyListener::stopTrigSilence() {
	GPIOA_OUTDR |= (((uint32_t) 1) << 2);      // PA2 = high z
}


int main() {
    interruptInit();
    MyListener myListener;
    Timer::init(myListener, 100_msForTimer);
	while (true)
		asm volatile ("wfi");
}
